import { Node } from "typescript";
import { Library } from "./library";
import { DecoratorData } from "./utils";
export declare namespace Decorator {
    type ClassID = 'SWAGGER_TAGS' | 'CLASS_PATH' | MethodAndCLassID;
    type MethodAndCLassID = 'REQUEST_ACCEPT' | 'RESPONSE_EXAMPLE' | 'RESPONSE_DESCRIPTION' | 'REQUEST_CONSUMES' | 'RESPONSE_PRODUCES' | 'SWAGGER_HIDDEN';
    type MethodHttpVerbID = 'ALL' | 'GET' | 'POST' | 'PUT' | 'DELETE' | 'PATCH' | 'OPTIONS' | 'HEAD';
    type MethodID = 'METHOD_PATH' | MethodHttpVerbID | MethodAndCLassID;
    type ParameterID = ParameterServerID | 'IS_INT' | 'IS_LONG' | 'IS_FlOAT' | 'IS_DOUBLE';
    type ParameterServerID = 'SERVER_CONTEXT' | 'SERVER_PARAMS' | 'SERVER_QUERY' | 'SERVER_FORM' | 'SERVER_BODY' | 'SERVER_HEADERS' | 'SERVER_COOKIES' | 'SERVER_PATH_PARAMS' | 'SERVER_FILE_PARAM' | 'SERVER_FILES_PARAM';
    type ID = ClassID | MethodID | ParameterID;
    class RepresentationResolver {
        protected id: ID;
        protected representationConfig: Array<RepresentationConfig>;
        constructor(id: ID, representationConfig: RepresentationConfig | Array<RepresentationConfig>);
        getNames(): string[];
        isPresentOnNode(node: Node): boolean;
        getDecoratorPropertyValueAsArray(decorator: DecoratorData, property?: Property): undefined | Array<any>;
        getDecoratorPropertyValueAsItem(decorator: DecoratorData, property?: Property): undefined | any;
        getDecoratorPropertyValue(decorator: DecoratorData, property: Property): any | Array<any> | undefined;
        getPropertyByType(representationName: string, type?: PropertyType): Property | undefined;
        getPropertiesByTypes(representationName: string, types: Array<PropertyType>): Record<PropertyType, Property>;
        buildRepresentationConfigFromNode(node: Node): RepresentationConfig;
        private static fillProperty;
    }
    type PropertyType = 'PAYLOAD' | 'STATUS_CODE' | 'DESCRIPTION' | 'OPTIONS' | 'SIMPLE' | 'TYPE';
    interface Property {
        /**
         * Default: 'SIMPLE'
         */
        type?: PropertyType;
        /**
         * Default: 'argument'
         */
        declaredAs?: 'argument' | 'typeArgument';
        /**
         * Default: 'one'
         */
        amount?: 'one' | 'all';
        /**
         * Default: 0
         */
        position?: number;
    }
    type Representation = Record<ID, RepresentationConfig | Array<RepresentationConfig>>;
    interface RepresentationConfig {
        name: string;
        decorator?: DecoratorData;
        decorators?: Array<DecoratorData>;
        properties?: Array<Property>;
    }
    type ConfigLibrary = Library | Array<Library> | Record<Library, Array<ID> | ID | Record<ID, boolean>>;
    interface Config {
        useLibrary?: ConfigLibrary;
        useBuildIn?: boolean | Array<ID> | Record<ID, boolean> | ID;
        override?: Representation;
    }
    function getRepresentationHandler(id: ID, map?: Config): RepresentationResolver;
}
